module IsoDoc
  module NIST
    class WordConvert < IsoDoc::WordConvert
      def toc_insert(docxml, level)
        insertion = docxml.at("//div[h1 = 'Executive Summary']/"\
                              "preceding-sibling::div[h1][1]") ||
        docxml.at("//div[@class = 'WordSection2']/child::*[last()]")
        if docxml.at("//p[@class = 'TableTitle']")
          insertion.next = make_TableWordToC(docxml)
          insertion.next = %{<p class="TOCTitle">List of Tables</p>}
        end
        if docxml.at("//p[@class = 'FigureTitle']")
          insertion.next = make_FigureWordToC(docxml)
          insertion.next = %{<p class="TOCTitle">List of Figures</p>}
        end
        if docxml.at("//p[@class = 'h1Annex']")
          insertion.next = make_AppendixWordToC(docxml)
          insertion.next = %{<p class="TOCTitle">List of Appendices</p>}
        end
        insertion.next = make_WordToC(docxml, level)
        insertion.next = %{<p class="TOCTitle" style="page-break-before:
        always;">Table of Contents</p>}
        docxml
      end

      WORD_TOC_APPENDIX_PREFACE1 = <<~TOC.freeze
      <span lang="EN-GB"><span
        style='mso-element:field-begin'></span><span
        style='mso-spacerun:yes'>&#xA0;</span>TOC
        \\h \\z \\t &quot;h1Annex,1,h2Annex,2,h3Annex,3&quot; <span
        style='mso-element:field-separator'></span></span>
      TOC

      WORD_TOC_TABLE_PREFACE1 = <<~TOC.freeze
      <span lang="EN-GB"><span
        style='mso-element:field-begin'></span><span
        style='mso-spacerun:yes'>&#xA0;</span>TOC
        \\h \\z \\t &quot;TableTitle,tabletitle&quot; <span
        style='mso-element:field-separator'></span></span>
      TOC

      WORD_TOC_FIGURE_PREFACE1 = <<~TOC.freeze
      <span lang="EN-GB"><span
        style='mso-element:field-begin'></span><span
        style='mso-spacerun:yes'>&#xA0;</span>TOC
        \\h \\z \\t &quot;FigureTitle,figuretitle&quot; <span
        style='mso-element:field-separator'></span></span>
      TOC

      def header_strip(h)
        h = h.to_s.gsub(/<\/?p[^>]*>/, "").
          gsub(%r{<a [^>]+>[^<]+</a><aside>.*</aside>}m, "")
        super
      end

      def make_TableWordToC(docxml)
        toc = ""
        docxml.xpath("//p[@class = 'TableTitle']").each do |h|
          toc += word_toc_entry(1, header_strip(h))
        end
        toc.sub(/(<p class="MsoToc1">)/,
                %{\\1#{WORD_TOC_TABLE_PREFACE1}}) +  WORD_TOC_SUFFIX1
      end

      def make_FigureWordToC(docxml)
        toc = ""
        docxml.xpath("//p[@class = 'FigureTitle']").each do |h|
          toc += word_toc_entry(1, header_strip(h))
        end
        toc.sub(/(<p class="MsoToc1">)/,
                %{\\1#{WORD_TOC_FIGURE_PREFACE1}}) +  WORD_TOC_SUFFIX1
      end

      def make_AppendixWordToC(docxml)
        toc = ""
        docxml.xpath("//p[@class = 'h1Annex'] | //p[@class = 'h2Annex'] | "\
                     "p[@class = 'h3Annex']").each do |h|
          toc += word_toc_entry(h["class"][1].to_i, header_strip(h))
        end
        toc.sub(/(<p class="MsoToc1">)/,
                %{\\1#{WORD_TOC_APPENDIX_PREFACE1}}) +  WORD_TOC_SUFFIX1
      end
    end
  end
end

